package kr.co.vividnext.sodalive.admin.audition

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('ADMIN')")
@RequestMapping("/admin/audition")
class AdminAuditionController(private val service: AdminAuditionService) {
    @PostMapping
    fun createAudition(
        @RequestPart("image") image: MultipartFile,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.createAudition(image, requestString), "등록되었습니다.")

    @PutMapping
    fun updateAudition(
        @RequestPart("image", required = false) image: MultipartFile? = null,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.updateAudition(image, requestString), "수정되었습니다.")

    @GetMapping
    fun getAuditionList(pageable: Pageable) = ApiResponse.ok(
        service.getAuditionList(
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )
    )

    @GetMapping("/{id}")
    fun getAuditionDetail(@PathVariable id: Long) = ApiResponse.ok(
        service.getAuditionDetail(auditionId = id)
    )
}
