package kr.co.vividnext.sodalive.admin.audition.role

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('ADMIN')")
@RequestMapping("/admin/audition/role")
class AdminAuditionRoleController(private val service: AdminAuditionRoleService) {
    @PostMapping
    fun createAuditionRole(
        @RequestPart("image") image: MultipartFile,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.createAuditionRole(image, requestString), "등록되었습니다.")

    @PutMapping
    fun updateAuditionRole(
        @RequestPart("image", required = false) image: MultipartFile? = null,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.updateAuditionRole(image, requestString), "수정되었습니다.")

    @GetMapping("/{id}")
    fun getAuditionRoleDetail(@PathVariable id: Long) = ApiResponse.ok(
        service.getAuditionRoleDetail(auditionRoleId = id)
    )

    @GetMapping("/{id}/applicant")
    fun getAuditionApplicantList(
        @PathVariable id: Long,
        pageable: Pageable
    ) = ApiResponse.ok(
        service.getAuditionApplicantList(
            auditionRoleId = id,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )
    )
}
