package kr.co.vividnext.sodalive.admin.member

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/admin/member")
@PreAuthorize("hasRole('ADMIN')")
class AdminMemberController(private val service: AdminMemberService) {
    @PostMapping("/password/reset")
    fun resetPassword(@RequestBody request: ResetPasswordRequest) = ApiResponse.ok(
        service.resetPassword(request = request),
        "비밀번호가 초기화 되었습니다."
    )

    @PutMapping
    fun updateMember(@RequestBody request: UpdateMemberRequest) = ApiResponse.ok(
        service.updateMember(request = request),
        "수정되었습니다."
    )

    @GetMapping("/list")
    fun getMemberList(pageable: Pageable) = ApiResponse.ok(service.getMemberList(pageable))

    @GetMapping("/search")
    fun searchMember(
        @RequestParam(value = "search_word") searchWord: String,
        pageable: Pageable
    ) = ApiResponse.ok(service.searchMember(searchWord, pageable))

    @GetMapping("/creator/all/list")
    fun getCreatorAllList() = ApiResponse.ok(service.getCreatorAllList())

    @GetMapping("/creator/list")
    fun getCreatorList(pageable: Pageable) = ApiResponse.ok(service.getCreatorList(pageable))

    @GetMapping("/creator/search")
    fun searchCreator(
        @RequestParam(value = "search_word") searchWord: String,
        pageable: Pageable
    ) = ApiResponse.ok(service.searchCreator(searchWord, pageable))
}
