package kr.co.vividnext.sodalive.content.main

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.order.OrderService
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/audio-content/main")
class AudioContentMainController(
    private val service: AudioContentMainService,
    private val orderService: OrderService
) {
    @GetMapping("/new-content-upload-creator")
    fun newContentUploadCreatorList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getNewContentUploadCreatorList(
                memberId = member.id!!,
                isAdult = member.auth != null
            )
        )
    }

    @GetMapping("/banner-list")
    fun getMainBannerList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getAudioContentMainBannerList(
                memberId = member.id!!,
                isAdult = member.auth != null
            )
        )
    }

    @GetMapping("/order-list")
    fun getMainOrderList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            orderService.getAudioContentMainOrderList(
                memberId = member.id!!,
                limit = 20
            )
        )
    }

    @GetMapping("/new")
    fun getNewContentByTheme(
        @RequestParam("theme") theme: String,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getNewContentByTheme(
                theme,
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member,
                pageable
            )
        )
    }

    @GetMapping("/theme")
    fun getThemeList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getThemeList(isAdult = member.auth != null))
    }

    @GetMapping("/new/all")
    fun getNewContentAllByTheme(
        @RequestParam("isFree", required = false) isFree: Boolean? = null,
        @RequestParam("theme") theme: String,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getNewContentFor2WeeksByTheme(
                isFree = isFree ?: false,
                theme = theme,
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member = member,
                pageable = pageable
            )
        )
    }

    @GetMapping("/curation-list")
    fun getCurationList(
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getAudioContentCurationListWithPaging(
                memberId = member.id!!,
                isAdult = member.auth != null && (isAdultContentVisible ?: true),
                contentType = contentType ?: ContentType.ALL,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }
}
