package kr.co.vividnext.sodalive.content.main.banner

import kr.co.vividnext.sodalive.event.EventItem
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service

@Service
class AudioContentBannerService(
    private val repository: AudioContentBannerRepository,
    private val blockMemberRepository: BlockMemberRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getBannerList(tabId: Long, memberId: Long, isAdult: Boolean): List<GetAudioContentBannerResponse> {
        return repository.getAudioContentMainBannerList(tabId, isAdult)
            .filter {
                if (it.type == AudioContentBannerType.CREATOR && it.creator != null) {
                    !blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = it.creator!!.id!!)
                } else if (it.type == AudioContentBannerType.SERIES && it.series != null) {
                    !blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = it.series!!.member!!.id!!)
                } else {
                    true
                }
            }
            .map {
                GetAudioContentBannerResponse(
                    type = it.type,
                    thumbnailImageUrl = "$imageHost/${it.thumbnailImage}",
                    eventItem = if (it.type == AudioContentBannerType.EVENT && it.event != null) {
                        EventItem(
                            id = it.event!!.id!!,
                            thumbnailImageUrl = if (!it.event!!.thumbnailImage.startsWith("https://")) {
                                "$imageHost/${it.event!!.thumbnailImage}"
                            } else {
                                it.event!!.thumbnailImage
                            },
                            detailImageUrl = if (
                                it.event!!.detailImage != null &&
                                !it.event!!.detailImage!!.startsWith("https://")
                            ) {
                                "$imageHost/${it.event!!.detailImage}"
                            } else {
                                it.event!!.detailImage
                            },
                            popupImageUrl = null,
                            link = it.event!!.link
                        )
                    } else {
                        null
                    },
                    creatorId = if (it.type == AudioContentBannerType.CREATOR && it.creator != null) {
                        it.creator!!.id
                    } else {
                        null
                    },
                    seriesId = if (it.type == AudioContentBannerType.SERIES && it.series != null) {
                        it.series!!.id
                    } else {
                        null
                    },
                    link = it.link
                )
            }
    }
}
