package kr.co.vividnext.sodalive.content.main.tab.alarm

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.main.GetNewContentAllResponse
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service
import java.time.DayOfWeek
import java.time.LocalDateTime
import java.time.temporal.TemporalAdjusters

@Service
class AudioContentMainTabAlarmService(
    private val bannerService: AudioContentBannerService,
    private val contentRepository: AudioContentRepository,
    private val rankingService: RankingService,
    private val eventService: EventService,
    private val curationRepository: AudioContentCurationQueryRepository
) {
    fun fetchData(member: Member): GetContentMainTabAlarmResponse {
        val isAdult = member.auth != null
        val memberId = member.id!!

        val contentBannerList = bannerService.getBannerList(
            tabId = 4,
            memberId = memberId,
            isAdult = isAdult
        )

        val alarmThemeList = listOf("모닝콜", "슬립콜", "알람")
        val newAlarmContentList = contentRepository.findAlarmContentByTheme(
            memberId = memberId,
            theme = alarmThemeList,
            isAdult = isAdult,
            limit = 10
        )

        // 주간 랭킹 기간
        val currentDateTime = LocalDateTime.now()
        val startDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusWeeks(1)
            .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY))
        val endDate = startDate
            .plusDays(6)

        val rankAlarmContentList = rankingService.getContentRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = startDate,
            endDate = endDate,
            theme = alarmThemeList[0]
        )

        val eventBannerList = eventService.getEventList(isAdult = isAdult)
        val curationList = curationRepository.findByContentMainTabId(tabId = 4, isAdult = isAdult)
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = contentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult
                    )
                )
            }

        return GetContentMainTabAlarmResponse(
            contentBannerList = contentBannerList,
            alarmThemeList = alarmThemeList,
            newAlarmContentList = newAlarmContentList,
            rankAlarmContentList = rankAlarmContentList,
            eventBannerList = eventBannerList,
            curationList = curationList
        )
    }

    fun fetchAlarmContentByTheme(
        theme: String,
        member: Member,
        offset: Long,
        limit: Long
    ): GetNewContentAllResponse {
        val alarmThemeList = if (theme.isNotBlank()) {
            listOf(theme)
        } else {
            listOf("모닝콜", "슬립콜", "알람")
        }

        val memberId = member.id!!
        val isAdult = member.auth != null

        val totalCount = contentRepository.totalAlarmCountByTheme(
            memberId = memberId,
            theme = alarmThemeList,
            isAdult = isAdult
        )

        val items = contentRepository.findAlarmContentByTheme(
            memberId = memberId,
            theme = alarmThemeList,
            isAdult = isAdult,
            offset = offset,
            limit = limit
        )

        return GetNewContentAllResponse(totalCount, items)
    }
}
