package kr.co.vividnext.sodalive.content.main.tab.free

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/v2/audio-content/main/free")
class AudioContentMainTabFreeController(private val service: AudioContentMainTabFreeService) {
    @GetMapping
    fun fetchContentMainFree(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.fetchData(member))
    }

    @GetMapping("/introduce-creator")
    fun getIntroduceCreator(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getIntroduceCreator(
                member,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/new-content-by-theme")
    fun getNewContentByTheme(
        @RequestParam("theme") theme: String,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getNewContentByTheme(
                theme,
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/popular-content-by-creator")
    fun getPopularContentByCreator(
        @RequestParam creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getPopularContentByCreator(
                creatorId = creatorId,
                isAdult = member.auth != null
            )
        )
    }
}
