package kr.co.vividnext.sodalive.content.main.tab.free

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.AudioContentMainTabRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.content.main.tab.RecommendSeriesRepository
import kr.co.vividnext.sodalive.content.theme.AudioContentThemeQueryRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service

@Service
class AudioContentMainTabFreeService(
    private val repository: AudioContentMainTabRepository,
    private val bannerService: AudioContentBannerService,
    private val recommendSeriesRepository: RecommendSeriesRepository,
    private val curationRepository: AudioContentCurationQueryRepository,
    private val contentRepository: AudioContentRepository,
    private val rankingService: RankingService,
    private val audioContentRepository: AudioContentRepository,
    private val audioContentThemeRepository: AudioContentThemeQueryRepository
) {
    fun fetchData(member: Member): GetContentMainTabFreeResponse {
        val isAdult = member.auth != null
        val memberId = member.id!!
        val tabId = 7L

        val contentBannerList = bannerService.getBannerList(
            tabId = tabId,
            memberId = memberId,
            isAdult = isAdult
        )

        val introduceCreator = curationRepository.findByContentMainTabIdAndTitle(
            tabId = tabId,
            title = "크리에이터 소개",
            isAdult = isAdult
        )
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = contentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult
                    )
                )
            }

        val recommendSeriesList = recommendSeriesRepository.getRecommendSeriesList(isAdult = isAdult)

        val themeList = audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult, isFree = true)
        val newFreeContentList = if (themeList.isNotEmpty()) {
            audioContentRepository.findByTheme(
                memberId = member.id!!,
                isAdult = member.auth != null,
                contentType = ContentType.ALL,
                offset = 0,
                limit = 10,
                isFree = true
            )
        } else {
            emptyList()
        }

        val creatorList = repository.findCreatorWithHasFreeContent(memberId, 4)
        val playCountRankContentList = if (creatorList.isNotEmpty()) {
            rankingService.fetchFreeContentByCreatorIdTop4(creatorList[0].creatorId, isAdult)
        } else {
            emptyList()
        }

        val curationList = curationRepository.findByContentMainTabId(tabId = tabId, isAdult = isAdult)
            .filter { it.title != "크리에이터 소개" }
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = contentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult
                    )
                )
            }

        return GetContentMainTabFreeResponse(
            contentBannerList = contentBannerList,
            introduceCreator = if (introduceCreator.isNotEmpty()) {
                introduceCreator[0]
            } else {
                null
            },
            recommendSeriesList = recommendSeriesList,
            themeList = themeList,
            newFreeContentList = newFreeContentList,
            creatorList = creatorList,
            playCountRankContentList = playCountRankContentList,
            curationList = curationList
        )
    }

    fun getIntroduceCreator(member: Member, offset: Long, limit: Long): List<GetAudioContentMainItem> {
        val isAdult = member.auth != null
        val memberId = member.id!!

        val introduceCreatorCuration = curationRepository.findByContentMainTabIdAndTitle(
            tabId = 7L,
            title = "크리에이터 소개",
            isAdult = isAdult
        )

        return if (introduceCreatorCuration.isNotEmpty()) {
            contentRepository.findAudioContentByCurationIdV2(
                curationId = introduceCreatorCuration[0].id!!,
                memberId = memberId,
                isAdult = isAdult,
                offset = offset,
                limit = limit
            )
        } else {
            emptyList()
        }
    }

    fun getNewContentByTheme(
        theme: String,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        return audioContentRepository.findByTheme(
            memberId = member.id!!,
            theme = if (theme.isNotBlank()) {
                listOf(theme)
            } else {
                emptyList()
            },
            isAdult = member.auth != null && isAdultContentVisible,
            contentType = contentType,
            offset = offset,
            limit = limit,
            isFree = true
        )
    }

    fun getPopularContentByCreator(creatorId: Long, isAdult: Boolean): List<GetAudioContentRankingItem> {
        return rankingService.fetchFreeContentByCreatorIdTop4(creatorId, isAdult)
    }
}
