package kr.co.vividnext.sodalive.content.main.tab.home

import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.notice.ServiceNoticeService
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service
import java.time.DayOfWeek
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.time.temporal.TemporalAdjusters

@Service
class AudioContentMainTabHomeService(
    private val noticeService: ServiceNoticeService,
    private val bannerService: AudioContentBannerService,
    private val rankingService: RankingService,
    private val eventService: EventService
) {
    fun fetchData(member: Member): GetContentMainTabHomeResponse {
        // 주간 랭킹 기간
        val currentDateTime = LocalDateTime.now()
        val startDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusWeeks(1)
            .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY))
        val endDate = startDate
            .plusDays(6)

        val startDateFormatter = DateTimeFormatter.ofPattern("yyyy년 MM월 dd일")
        val endDateFormatter = DateTimeFormatter.ofPattern("MM월 dd일")

        val formattedLastMonday = startDate.format(startDateFormatter)
        val formattedLastSunday = endDate.format(endDateFormatter)

        // 최근 공지사항
        val latestNotice = noticeService.getLatestNotice()

        // 메인 배너 (홈)
        val contentBannerList = bannerService.getBannerList(
            tabId = 1,
            memberId = member.id!!,
            isAdult = member.auth != null
        )

        // 인기 크리에이터
        val rankCreatorList = rankingService.getCreatorRanking(
            memberId = member.id!!,
            rankingDate = "$formattedLastMonday ~ $formattedLastSunday"
        )

        // 인기 시리즈
        val rankSeriesList = rankingService.getSeriesRanking(
            memberId = member.id!!,
            isAdult = member.auth != null,
            startDate = startDate.minusDays(1),
            endDate = endDate
        )

        // 인기 콘텐츠
        val rankContentList = rankingService.getContentRanking(
            memberId = member.id!!,
            isAdult = member.auth != null,
            startDate = startDate.minusDays(1),
            endDate = endDate
        )

        // 이벤트 배너
        val eventBannerList = eventService.getEventList(isAdult = member.auth != null)

        val contentRankCreatorList = rankingService.fetchCreatorBySellContentCountRankTop20(
            memberId = member.id!!,
            startDate = startDate.minusDays(1),
            endDate = endDate
        )

        val salesCountRankContentList = if (contentRankCreatorList.isNotEmpty()) {
            rankingService.fetchCreatorContentBySalesCountTop4(
                creatorId = contentRankCreatorList[0].creatorId,
                isAdult = member.auth != null
            )
        } else {
            emptyList()
        }

        return GetContentMainTabHomeResponse(
            latestNotice = latestNotice,
            bannerList = contentBannerList,
            rankCreatorList = rankCreatorList,
            rankSeriesList = rankSeriesList,
            rankSortTypeList = listOf("매출", "댓글", "좋아요"),
            rankContentList = rankContentList,
            eventBannerList = eventBannerList,
            contentRankCreatorList = contentRankCreatorList,
            salesCountRankContentList = salesCountRankContentList
        )
    }

    fun getPopularContentByCreator(creatorId: Long, isAdult: Boolean): List<GetAudioContentRankingItem> {
        return rankingService.fetchCreatorContentBySalesCountTop4(
            creatorId = creatorId,
            isAdult = isAdult
        )
    }
}
