package kr.co.vividnext.sodalive.content.main.tab.series

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/v2/audio-content/main/series")
class AudioContentMainTabSeriesController(private val service: AudioContentMainTabSeriesService) {
    @GetMapping
    fun fetchContentMainSeries(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.fetchData(member))
    }

    @GetMapping("/original")
    fun getOriginalAudioDramaList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getOriginalAudioDramaList(
                memberId = member.id!!,
                isAdult = member.auth != null,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/completed-monthly-rank")
    fun getRankMonthlyCompletedSeriesList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getRankMonthlyCompletedSeriesList(
                memberId = member.id!!,
                isAdult = member.auth != null,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/recommend-by-genre")
    fun getRecommendSeriesListByGenre(
        @RequestParam genreId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getRecommendSeriesListByGenre(
                genreId,
                memberId = member.id!!,
                isAdult = member.auth != null
            )
        )
    }

    @GetMapping("/recommend-series-by-creator")
    fun getRecommendSeriesByCreator(
        @RequestParam creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getRecommendSeriesByCreator(
                creatorId = creatorId,
                isAdult = member.auth != null
            )
        )
    }
}
