package kr.co.vividnext.sodalive.content.theme

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.SortType
import kr.co.vividnext.sodalive.content.theme.content.GetContentByThemeResponse
import kr.co.vividnext.sodalive.member.Member
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AudioContentThemeService(
    private val queryRepository: AudioContentThemeQueryRepository,
    private val contentRepository: AudioContentRepository
) {
    @Transactional(readOnly = true)
    fun getThemes(): List<GetAudioContentThemeResponse> {
        return queryRepository.getActiveThemes()
    }

    @Transactional(readOnly = true)
    fun getContentByTheme(
        themeId: Long,
        sortType: SortType,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        offset: Long,
        limit: Long
    ): GetContentByThemeResponse {
        val theme = queryRepository.findThemeByIdAndActive(themeId)
            ?: throw SodaException("잘못된 요청입니다.")

        val totalCount = contentRepository.totalCountByTheme(
            memberId = member.id!!,
            theme = listOf(theme.theme),
            isAdult = member.auth != null && isAdultContentVisible,
            contentType = contentType
        )

        val items = contentRepository.findByTheme(
            memberId = member.id!!,
            theme = listOf(theme.theme),
            sortType = sortType,
            isAdult = member.auth != null && isAdultContentVisible,
            contentType = contentType,
            offset = offset,
            limit = limit
        )

        return GetContentByThemeResponse(
            theme = theme.theme,
            totalCount = totalCount,
            items = items
        )
    }
}
