package kr.co.vividnext.sodalive.creator.admin.calculate

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@PreAuthorize("hasRole('CREATOR')")
@RequestMapping("/creator-admin/calculate")
class CreatorAdminCalculateController(private val service: CreatorAdminCalculateService) {
    @GetMapping("/live")
    fun getCalculateLive(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(service.getCalculateLive(startDateStr, endDateStr, member))
    }

    @GetMapping("/content-list")
    fun getCalculateContentList(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(
            service.getCalculateContentList(
                startDateStr,
                endDateStr,
                member.id!!,
                pageable.offset,
                pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/cumulative-sales-by-content")
    fun getCumulativeSalesByContent(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getCumulativeSalesByContent(member.id!!, pageable.offset, pageable.pageSize.toLong()))
    }

    @GetMapping("/content-donation-list")
    fun getCalculateContentDonationList(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")
        ApiResponse.ok(
            service.getCalculateContentDonationList(
                startDateStr,
                endDateStr,
                member.id!!,
                pageable.offset,
                pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/community-post")
    fun getCalculateCommunityPost(
        @RequestParam startDateStr: String,
        @RequestParam endDateStr: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getCalculateCommunityPost(
                startDateStr,
                endDateStr,
                member.id!!,
                pageable.offset,
                pageable.pageSize.toLong()
            )
        )
    }
}
