package kr.co.vividnext.sodalive.creator.admin.content.series

import kr.co.vividnext.sodalive.common.SodaException

data class CreateSeriesRequest(
    val title: String,
    val introduction: String,
    val publishedDaysOfWeek: MutableSet<SeriesPublishedDaysOfWeek>,
    val keyword: String,
    val genreId: Long = 0,
    val isAdult: Boolean = false,
    val writer: String? = null,
    val studio: String? = null
) {
    fun toSeries(): Series {
        validate()

        return Series(
            title = title,
            introduction = introduction,
            writer = writer,
            studio = studio,
            publishedDaysOfWeek = publishedDaysOfWeek,
            isAdult = isAdult
        )
    }

    private fun validate() {
        if (title.isBlank()) throw SodaException("시리즈 제목을 입력하세요")
        if (introduction.isBlank()) throw SodaException("시리즈 소개를 입력하세요")
        if (keyword.isBlank()) throw SodaException("시리즈를 설명할 수 있는 키워드를 입력하세요")
        if (genreId <= 0) throw SodaException("올바른 장르를 선택하세요")
        if (publishedDaysOfWeek.isEmpty()) throw SodaException("시리즈 연재요일을 선택하세요")
        if (publishedDaysOfWeek.contains(SeriesPublishedDaysOfWeek.RANDOM) && publishedDaysOfWeek.size > 1) {
            throw SodaException("랜덤과 연재요일 동시에 선택할 수 없습니다.")
        }
    }
}
