package kr.co.vividnext.sodalive.member.stipulation

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.stipulation.StipulationIds.PRIVACY_POLICY_ID
import kr.co.vividnext.sodalive.member.stipulation.StipulationIds.TERMS_OF_SERVICE_ID
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
@Transactional(readOnly = true)
class StipulationService(private val repository: StipulationRepository) {
    fun enrollment(request: StipulationDto) {
        repository.save(request.toEntity())
    }

    fun getTermsOfService(): StipulationDto {
        val stipulation = repository.findByIdOrNull(TERMS_OF_SERVICE_ID)
            ?: throw SodaException("잘못된 요청입니다\n앱 종료 후 다시 시도해 주세요.")

        return StipulationDto(stipulation)
    }

    fun getPrivacyPolicy(): StipulationDto {
        val stipulation = repository.findByIdOrNull(PRIVACY_POLICY_ID)
            ?: throw SodaException("잘못된 요청입니다\n앱 종료 후 다시 시도해 주세요.")

        return StipulationDto(stipulation)
    }

    @Transactional
    fun modify(request: StipulationModifyRequest) {
        val stipulation = repository.findByIdOrNull(request.id)
            ?: throw SodaException("잘못된 요청입니다\n앱 종료 후 다시 시도해 주세요.")

        stipulation.description = request.description
    }
}
